require 'logger'
require 'rt_client'

if (RUBY_VERSION.to_f > 2.9)

  module URI
    class << self
      def escape(str)
        alpha = "a-zA-Z"
        alnum = "#{alpha}\\d"
        unreserved = "\\-_.!~*'()#{alnum}"
        reserved = ";/?:@&=+$,\\[\\]"
        unsafe = Regexp.new("[^#{unreserved}#{reserved}]")
        str.gsub(unsafe) do
          us = $&
          tmp = ''
          us.each_byte do |uc|
            tmp << sprintf('%%%02X', uc)
          end
          tmp
        end.force_encoding(Encoding::US_ASCII)
      end
    end
  end

  class RT_Client
    # helper to convert responses from RT REST to a hash
    def response_to_h(resp) # :nodoc:
      if resp.class.name == 'String'
        resp = resp.gsub(/RT\/\d+\.\d+\.\d+\s\d{3}\s.*\n\n/,"") # toss the HTTP response
      else
        resp = resp.body.gsub(/RT\/\d+\.\d+\.\d+\s\d{3}\s.*\n\n/,"") # toss the HTTP response
      end

      # unfold folded fields
      # A newline followed by one or more spaces is treated as a
      # single space
      resp.gsub!(/\n +/, " ")

      #replace CF spaces with underscores
      while resp.match(/CF\.\{[\w_ ]*[ ]+[\w ]*\}/)
        resp.gsub!(/CF\.\{([\w_ ]*)([ ]+)([\w ]*)\}/, 'CF.{\1_\3}')
      end
      return {:error => resp }  if (resp =~ /^#/ and resp =~/does not exist\.$/) or (resp =~ /Transaction \d+ is not related to Ticket/)

      # convert fields to key value pairs
      ret = {}
      resp.each_line do |ln|
        next unless ln =~ /^.+?:/
        ln_a = ln.split(/:/,2)
        ln_a.map! {|item| item.strip}
        ln_a[0].downcase!
        ret[ln_a[0]] = ln_a[1]
      end
      return ret
    end
  end

end

module PLM
  module Model
    class Error
      module ClassMixin
        def send_error(params={})
          e       = params.fetch(:error, StandardError.new("No reason issued"))
          session = params.fetch(:session,{})
          begin
            error_manager = PLM::Model::Config::CONFIG[:ticket]
            error_manager[:type] = 'ticket'
          rescue
            error_manager = {type: 'stderr'}
          end
          case error_manager[:type]
          when 'ticket'
            send_ticket config: error_manager,
            message: e.message,
            reason:  e.class.name,
            session: session
          when 'stderr'
            logger message: e.message,
            reason:       e.class.name,
            session:      session
          else
            puts "No manager set: Reason: #{e.class.name} Message: #{e.message} for #{session}"
          end
        end

        def send_ticket(params={})
          ticket_config = params.fetch(:config, PLM::Model::Config::CONFIG[:ticket])
          reason        = params.fetch(:reason,'PLM::Model::Error')
          message       = params.fetch(:message ,'No reason issued')
          session       = params.fetch(:session,{})

          credentials = ticket_config[:credentials]
          credentials[:user]=credentials[:username] if credentials.has_key?(:username)
          credentials[:pass]=credentials[:password] if credentials.has_key?(:password)
          rt = RT_Client.new(credentials)

          if rt.show(1)
            message = message+"\npour une connexion venant de :\n#{session}" unless session == {}
            tickets = rt.query(query: "Subject LIKE \"#{reason}\" and (Status = 'new' or Status = 'open')")
            if tickets.size == 0
              attrs = {
                Subject: reason,
                Queue:  ticket_config[:queue],
                #'Owner'      => ticket_config[:owner],
                Text:   message
              }

              attrs[:Cc] = params[:Cc] if params.has_key?(:Cc)

              rt.create(attrs)
            else

              attrs = {
                id: tickets.last['id'].delete('ticket/'),
                Text: message
              }

              rt.comment(attrs)
            end
          end

        end

        def resolve_ticket(params={})
          ticket_config = params.fetch(:config, PLM::Model::Config::CONFIG[:ticket])
          reason        = params.fetch(:reason,'PLM::Model::Error')
          message       = params.fetch(:message ,'')

          credentials = ticket_config[:credentials]
          credentials[:user]=credentials[:username] if credentials.has_key?(:username)
          credentials[:pass]=credentials[:password] if credentials.has_key?(:password)
          rt = RT_Client.new(credentials)

          if rt.show(1)
            tickets = rt.query(query: "Subject LIKE \"#{reason}\" and (Status = 'new' or Status = 'open')")
            tickets.uniq.each do |ticket|

              id = ticket['id'].delete('ticket/')

              rt.comment({ id: id, Text: message }) unless message == ''

              rt.edit(id: id, Status: "resolved")
            end
          end
        end

        def logger(params={})
          reason  = params.fetch(:reason,'PLM::Model::Error')
          message = params.fetch(:message ,'No reason issued')
          session = params.fetch(:session,{})

          message = message+"\npour une connexion venant de :\n#{session}" unless session == {}
          logger  = Logger.new(STDERR)

          logger.error "Reason: #{reason} Message: #{message}"
          logger.close
        end
      end
    end
  end
end
